// FlashID.cpp : Defines the entry point for the application.
//
// FlashID.exe
// ADS# 720020-11981
//
// This program uses the new IOCTLs in the FlashFX Driver to stop the FlashFX Driver,
// Read the Serial Number and Device ID of the Flash Chip, and then resume the FlashFX Driver.
// The driver must be stopped when reading the Serial Number and Device ID so as to 
// prevent corrupting the flash.  It must then be resumed so the system can continue to operate
// normally.
//
// jnosek 9-28-2006

// cbadr 05-16-2008
// get size of ID returned at runtime
// reverse print order to reflect MSB and LSB correctly

#include "stdafx.h"
#include "windows.h"
#include "flash.h"

// ADS Product Number
#define ADS_NUMBER  TEXT("720020-11982")

#define DRIVER_NAME TEXT("FSH1:")

void printBytes(BYTE* b, int size);

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	BOOL bRet = FALSE;
	DWORD BytesRet = 0;
	
	HANDLE hPort = INVALID_HANDLE_VALUE;

	RETAILMSG(1,(TEXT("\r\nFlashID   %s\r\n"), ADS_NUMBER));

	// Open a handle to the Flash driver
	hPort = CreateFile ( DRIVER_NAME, 
                         GENERIC_WRITE | GENERIC_READ,
                         0,
                         NULL,
                         OPEN_EXISTING,
                         FILE_ATTRIBUTE_NORMAL,
                         NULL);
	
	// sanity check
	if (hPort == INVALID_HANDLE_VALUE) 
	{
		RETAILMSG(1,(TEXT("Could not open File %s.\r\n"),DRIVER_NAME));
		return FALSE;
	}
	else
		RETAILMSG(1,(TEXT("Opened File %s ...\r\n"),DRIVER_NAME));
	
	// Clear data
	BYTE a[1] = {0};
	BYTE b[8] = {0,0,0,0,0,0,0,0};
	
	// Stop the flash disk
	bRet = DeviceIoControl( hPort,
							IOCTL_STOP_FLASHFX_DISK,
							a, 
							0,
							a,
							0,
							&BytesRet,
							NULL);

	if(!bRet)
	{
		RETAILMSG(1,(TEXT("Stop FlashFX FAILED\r\n"), b));
		//return 0;
	}

	// read the 64 bit flash ID through new IOCTL
	RETAILMSG(1,(TEXT("ReadFlashID  New IOCTL: ")));
	bRet = DeviceIoControl(	hPort,
							IOCTL_READ_FLASH_SER_NUM,
							a,           // not used, but can't be NULL
							0,
							b,
							8,
							&BytesRet,   
							NULL);
	if(!bRet)
	{
		RETAILMSG(1,(TEXT("FAILED  "), b));
		//return 0;
	}
		
	printBytes(b, BytesRet);
	RETAILMSG(1,(TEXT("\r\n")));


	// read the 64 bit flash ID through old IOCTL
	RETAILMSG(1,(TEXT("ReadFlashID  Old IOCTL: ")));
	bRet = DeviceIoControl(	hPort,
							IOCTL_READ_FLASH_ID,
							a,           // not used, but can't be NULL
							0,
							b,
							8,
							&BytesRet,
							NULL  
							);
	if(!bRet)
	{
		RETAILMSG(1,(TEXT("FAILED  "), b));
		//return 0;
	}
		
	printBytes(b, BytesRet);
	RETAILMSG(1,(TEXT("\r\n")));


	// read the 32 bit device ID through new IOCTL
	RETAILMSG(1,(TEXT("ReadDeviceID New IOCTL: ")));

	bRet = DeviceIoControl(	hPort,
							IOCTL_READ_FLASH_DEVICE_ID,
							a,           // not used, but can't be NULL
							0,
							b,
							4,
							&BytesRet,   
							NULL);
	if(!bRet)
	{
		RETAILMSG(1,(TEXT("FAILED  "), b));
		//return 0;
	}
		
	printBytes(b, BytesRet);
	RETAILMSG(1,(TEXT("\r\n")));
	
	// Resume the flash disk
	bRet = DeviceIoControl( hPort,
							IOCTL_RESUME_FLASHFX_DISK,
							a, 
							0,
							a,
							0,
							&BytesRet,
							NULL);
	if(!bRet)
	{
		RETAILMSG(1,(TEXT("Resume FlashFX FAILED\r\n")));
		//return 0;
	}
	
	
	// close the driver
	if(hPort != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hPort);
		hPort = INVALID_HANDLE_VALUE;
	}
	
	RETAILMSG(1,(TEXT("...Done\r\n")));

	// exit
	return 0;
}

// Used to print each byte of an id
void printBytes(BYTE* b, int size)
{
	BYTE* p = b + size - 1;
	for(int i = size - 1; i >= 0; i--)
	{
		RETAILMSG(1, (TEXT("%02X "), *p));
		p--;
	}
}
